/*
 * clock for root window.
 */

/*
 * - finish alarms set up
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <librootapp/librootapp.h>
#include "config.h"
#include "mem.h"

#define EVENTMASK VisibilityChangeMask | ExposureMask

/* Global varriables */
static int end_program = FALSE;

static char          *dpyname    = NULL;
static char          *geometry   = NULL;
static char          *mcolorname = NULL;
static char          *scolorname = NULL;
static char          *fontname   = NULL;
static int            ignbuffers = False;
static int            igncached  = False;
static int            ignwired   = False;
static struct mem_options mem_opts;
static unsigned long  cm = 0, cs = 0;


/* Command line options and config file entries */
static Option opttab[] = {
    {'h', NULL,      NULL, O_HELP,   NULL,        FALSE, "display help"},
    {'v', NULL,      NULL, O_VERS,   NULL,        FALSE, "display version"},
    {'d', "display", NULL, C_STRING, &dpyname,    FALSE, "display name"},
    {'g', "geom",    NULL, C_STRING, &geometry,   FALSE, "set geometry"},
    {'f', "font",    NULL, C_STRING, &fontname,   FALSE, "text font"},
    {'t', "color",   NULL, C_STRING, &mcolorname, FALSE, "graph & text color"},
    {'b', "color",   NULL, C_STRING, &scolorname, FALSE, "border color"},
#ifdef IGNORE_BUFFERS
    {'i', NULL,      NULL, C_BOOL,   &ignbuffers, FALSE, "ignore buffers"},
#endif
#ifdef IGNORE_CACHED
    {'i', NULL,      NULL, C_BOOL,   &igncached,  FALSE, "ignore cached"},
#endif
#ifdef IGNORE_WIRED
    {'i', NULL,      NULL, C_BOOL,   &ignwired,   FALSE, "ignore wired"},
#endif
    {0,   NULL,      NULL, C_NONE,   NULL,        FALSE, NULL}
};



/****** DISPLAYING FUNCTIONS ******/

void draw_load(RootApp *app, int mem, int swap) {
    int  h = font_height();
    int  cw, ch;

    ch = (h - 3) / 2;
    set_color(cm);
    root_print(app, "mem", app->wsize.width - 1 - text_width("mem"), 0);

    cw = mem * (app->wsize.width - 2) / 100;
    XFillRectangle(display, app->bgpixmap, gc,
            1, h+1, cw + 1, ch);

    cw = swap * (app->wsize.width - 2) / 100;
    XFillRectangle(display, app->bgpixmap, gc,
            1, app->wsize.height - 1 - ch, cw + 1, ch);

    set_color(cs);
    XDrawRectangle(display, app->bgpixmap, gc,
            0, h, app->wsize.width - 1, ch + 1);
    XDrawRectangle(display, app->bgpixmap, gc,
            0, app->wsize.height - 2 - ch, app->wsize.width - 1, ch + 1);
}


/****** PROCESSING FUNCTIONS ******/

void main_loop(RootApp *app) {
    XEvent  event;
    int     cont = TRUE;
    int     mem, swap;

    mem_getusage(&mem, &swap, &mem_opts);
    draw_load(app, mem, swap);
    display_app(app);
    while (cont) {
        nextevent_or_timeout(&event, 1000);
        clear_bg(app);
        mem_getusage(&mem, &swap, &mem_opts);
        draw_load(app, mem, swap);
        display_app(app);
        if (end_program) cont = FALSE;
    }
}


/****** SIGNALS ******/

void install_handler();


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void install_handler() {
    signal(SIGTERM, signals_handler);
    signal(SIGQUIT, signals_handler);
    signal(SIGINT,  signals_handler);
}


void manage_signals() {
    struct sigaction sa;

    sa.sa_handler = SIG_IGN;
#ifdef SA_NOCLDWAIT
    sa.sa_flags = SA_NOCLDWAIT;
#else
    sa.sa_flags = 0;
#endif
    sigemptyset(&sa.sa_mask);
    sigaction(SIGCHLD, &sa, NULL);
    install_handler();
}


void set_defaults() {
    mem_opts.ignore_buffers = ignbuffers;
    mem_opts.ignore_cached  = igncached;
    mem_opts.ignore_wired   = ignwired;
}


void set_app_size(RootApp *app) {
    int w = text_width("00:00:00");
    int h = 2 * (font_height() + 1);

    resize(app, geometry, 0, 0, w, h);
}


/****** MAIN ******/


int main(int argc, char *argv[]) {
    RootApp    *app;

    manage_signals();
    setlocale(LC_ALL, "");
    init_rootapp(PACKAGE, PROGRAM, VERSION, AUTHORS);
    parse_argv(opttab, argc, argv);

    set_defaults();
    mem_init();

    /* X11 init */
    open_x11(dpyname);
    if (mcolorname) cm = get_color(mcolorname);
    if (scolorname) cs = get_color(scolorname);
    if (fontname)  load_font(fontname);

    /* Creation */
    app = create_app(EVENTMASK, geometry);
    set_app_size(app);

    main_loop(app);

    /* Clean */
    close_app(app);
    close_x11();
    free_vars(opttab);

    return 0;
}
